@@ -2,6 +2,7 @@ import os
 import sys
 import runcmd
 import yaml
+
 curPath = os.path.abspath(os.path.dirname(__file__))
 rootPath = os.path.split(curPath)[0]
 sys.path.append(rootPath)
@@ -129,14 +130,24 @@ def load_yaml(data):
     return convertDepList
 
 
-# 清空文件夹及目录
-def del_file(path_data):
+# 删除文件
+def del_path(path):
     """
-    :param path_data: 文件路径
+    :param path: 文件路径
     :return: 无返回值
     """
     # os.listdir(path_data)#返回一个列表，里面是当前目录下面的所有东西的相对路径
-    shutil.rmtree(path_data)
+    os.remove(path)
+
+
+# 清空文件夹及目录
+def del_dir(path):
+    """
+    :param path: 文件路径
+    :return: 无返回值
+    """
+    # os.listdir(path_data)#返回一个列表，里面是当前目录下面的所有东西的相对路径
+    shutil.rmtree(path)
 
 
 # 清空或者创建一个新的目录
@@ -149,7 +160,7 @@ def create_file(path):
     if not os.path.exists(path):
         os.makedirs(path)
     else:
-        del_file(path)
+        del_dir(path)
         os.makedirs(path)
 
 
@@ -209,6 +220,20 @@ class RepoGit:
         """
         return str(self.repo.active_branch)
 
+    # 创建新分支
+    def create_branch(self, branch):
+        """
+        :return: 新分支名
+        """
+        return self.repo.create_head(branch)
+
+    # 推送分支到远端
+    def push_branch(self, branch):
+        """
+        :return: 新分支名
+        """
+        self.repo.remotes.origin.push(refspec=branch)
+
     # 获取当前工作目录的文件状态，是否有改动
     # True 表示有改动未提交 False表示没有改动
     def is_dirty(self):
@@ -395,13 +420,17 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
     """
     # 获取ymal 数据
     podfile_module_data = yaml_data(yaml_path)
-    dependenceList = podfile_module_data["dependencies"]
+    dependenceList = []
+    if "dependencies" in podfile_module_data:
+        local_dependenceList = podfile_module_data["dependencies"]
+        if local_dependenceList:
+            dependenceList = local_dependenceList
+
     before_branch = ''
     if "branch" in podfile_module_data:
         before_branch = podfile_module_data['branch']
     # 转换成模型数组
     conver_deplist = load_yaml(dependenceList)
-    # print("origin=======" + str(dependenceList))
 
     index = 0
     for a in conver_deplist:
@@ -439,43 +468,34 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
             if mo_re.result == 1:
                 module_dict = {"module": mo_re.module, "pod": mo_re.module, "path": modules_name + "/" + mo_re.module}
                 after_convert.append(module_dict)
+        local_module_data = {"version": "1.0.0", "branch": str(n_branch), "dependencies": after_convert}
+        save_dict_to_yaml(local_module_data, local_yaml_path)
+
     else:
         # 获取ymal 数据
-        podfile_module_data = yaml_data(local_yaml_path)
+        local_module_data = yaml_data(local_yaml_path)
         local_dependenceList = []
         conver_deplist = []
-        if podfile_module_data:
-
-            if "dependencies" in podfile_module_data:
-                local_dependenceList = podfile_module_data["dependencies"]
+        if local_module_data:
+            if "dependencies" in local_module_data:
+                local_dependenceList = local_module_data["dependencies"]
             if not local_dependenceList:
                 local_dependenceList = []
             # 转换成模型数组
             conver_deplist = load_yaml(local_dependenceList)
+        debugInfo(str([a.module for a in conver_deplist]))
+        debugInfo(str([a.module for a in branch_result]))
+        except_list = [
+            module for module in branch_result if ((module.result == 1) and (module.module not in [
+                pod.module for pod in conver_deplist]))]
+        debugInfo(str([a.module for a in except_list]))
+        if len(except_list) > 0:
+            for mo_re in except_list:
+                module_dict = {"module": mo_re.module, "pod": mo_re.pod, "path": modules_name + "/" + mo_re.module}
+                local_dependenceList.append(module_dict)
 
-        for yamlModel in branch_result:
-            contain = False
-            for cm in conver_deplist:
-                if cm.module == yamlModel.module:
-                    contain = True
-            if not contain and yamlModel.result == 1:
-                # 没有包含，添加一个
-                module = YamlModuleModel(module=yamlModel.module,
-                                         pod=yamlModel.pod,
-                                         version=None,
-                                         git=None,
-                                         branch=None,
-                                         tag=None,
-                                         path=None,
-                                         newtag=None,
-                                         configurations=None,
-                                         inhibit_warnings=False)
-                conver_deplist.append(module)
-        for a in conver_deplist:
-            module_dict = {"module": a.module, "pod": a.pod, "path": modules_name + "/" + a.module}
-            after_convert.append(module_dict)
-    local_module_data = {"version": "1.0.0", "branch": str(n_branch), "dependencies": after_convert}
-    save_dict_to_yaml(local_module_data, local_yaml_path)
+        local_module_data = {"version": "1.0.0", "branch": str(n_branch), "dependencies": local_dependenceList}
+        update_yaml(local_yaml_path, local_module_data)
 
 
 # 更新podfileModule文件
@@ -676,29 +696,28 @@ def new_branch(filepath, git, f_branch, f_tag, n_branch):
     git_create_branch = "git branch " + n_branch
     if f_tag and len(f_tag) > 0:
         git_create_branch += " " + f_tag
-    # checkout
-    git_checkout_command = "git checkout -b " + n_branch
-    # git push
-    git_push_command = "git push origin {0}".format(n_branch)
-    os.system(git_push_command)
     create_status = 0
     create_status += os.system(git_clone_command)
-    logging.info("执行 " + git_clone_command)
     # master分支下的版本号
     repoGit = RepoGit(proj_path=filepath)
     branchs = repoGit.get_branches()
+    debugInfo(str(branchs))
     os.chdir(filepath)
     if n_branch in branchs:
-        os.system(git_checkout_command)
-        logging.info(n_branch + "分支已存在")
+        debugInfo(n_branch + "分支已存在")
+        repoGit.repo.git.execute(['git', 'checkout', n_branch])
         return n_branch
-    create_status = os.system(git_create_branch)
-    create_status += os.system(git_checkout_command)
-    logging.info("新分支已经提交到master")
-    create_status += os.system(git_push_command)
-    if not create_status == 0:
-        logging.warning("新分支创建失败")
+    try:
+        # 创建并切换到新分支
+        new_b = repoGit.create_branch(n_branch)
+        new_b.checkout()
+        # 推送新分支到远程仓库
+        repoGit.push_branch(new_b)
+    except Exception as e:
+        print(str(e))
+        debugInfo("新分支创建失败")
         return ""
+    debugInfo("新分支创建成功")
     return n_branch
 
 
@@ -925,13 +944,16 @@ def release_branch(module_list, include_list, tag_path, c_path, f_branch):
 
     index = 0
     merge_result = []
-    bar = Bar('release...', fill="*", max=len(module_list))
-    for a in module_list:
+    invert_list = module_list
+    if len(include_list) > 0:
+        invert_list = [i for i in invert_list if i.module in include_list]
+
+    bar = Bar('release...\n', fill="*", max=len(invert_list))
+    for a in invert_list:
         bar.next()
         if not (a.branch and len(a.branch) > 0) and not (a.tag and len(a.tag) > 0):
             a.branch = f_branch
         if a.branch and len(a.branch) > 0:
-            if (len(include_list) > 0 and a.module in include_list) or len(include_list) == 0:
                 filename = get_filename(a.git)
                 module_path = tag_path + filename + "/"
                 create_tag = auto_release_path(module_path, a.git, a.pod, a.branch, a.new_tag)
@@ -1021,23 +1043,28 @@ class yhgit:
         file_name, file_ext = os.path.splitext(file_name)
         modules_name = "modules"
         modules_path = c_path + "modules/"
-        module_path = modules_path + file_name
-        result = auto_create_branch(module_path, git, n_branch, '', n_branch)
-        os.chdir(f_path)
         # 子目录，用来存放子模块仓库, 如果没有需要创建一个
         if not os.path.exists(modules_path):
             create_file(modules_path)
+
+        module_path = modules_path + file_name
+        result = auto_create_branch(module_path, git, n_branch, '', n_branch)
+        os.chdir(f_path)
+
         error = ""
         res = 1
+        newfile_name = ""
+        debugInfo("执行结果" + result)
         if result and len(result) > 0:
             # 新分支创建成功
             # 更新模块的名字{file_name}为{podspec}名字
-            newfile_name = ""
+            debugInfo(module_path)
             files = [filename for filename in os.listdir(f_path + "/" + module_path) if filename.endswith(".podspec")]
+            debugInfo(files)
             if len(files) > 0:
                 newfile_name = files[0]
                 newfile_name, file_ext = os.path.splitext(newfile_name)
-
+            debugInfo("执行结果" + newfile_name)
             newfile_path = modules_path + newfile_name
             # 判断是否有newfile_name对应的文件夹
             find = False
@@ -1050,19 +1077,26 @@ class yhgit:
             else:
                 # 删除旧的文件夹
                 if os.path.exists(f_path + "/" + module_path):
-                    shutil.rmtree(f_path + "/" + module_path)
+                    del_dir(f_path + "/" + module_path)
                 res = 1
                 error = "存在同名文件夹，新建分支失败，请手动切换"
         # 判断是否有PodfileModule文件夹
+        pre_path = f_path + "/"
+        debugInfo(pre_path)
+        localyamlPath_ = pre_path + 'PodfileLocal.yaml'
+        yamlPath_ = pre_path + 'PodfileModule.yaml'
         localyamlPath = c_path + 'PodfileLocal.yaml'
         yamlPath = c_path + 'PodfileModule.yaml'
-        if not os.path.getsize(localyamlPath):
-            shutil.rmtree(localyamlPath)
+        if os.path.exists(localyamlPath_) and os.path.getsize(localyamlPath) == 0:
+            del_path(localyamlPath_)
 
-        if not os.path.getsize(yamlPath):
-            shutil.rmtree(yamlPath)
+        debugInfo("yaml的路基" + yamlPath_ + "是否存在路基" + str(os.path.exists(yamlPath_)))
+        debugInfo("yamlPath的路基" + yamlPath + "文件大小" + str(os.path.getsize(yamlPath)))
 
-        if not os.path.exists(yamlPath):
+        if os.path.exists(yamlPath_) and os.path.getsize(yamlPath) == 0:
+            del_path(yamlPath_)
+
+        if not os.path.exists(yamlPath_):
             open(yamlPath, 'w').close()
             after_convert = []
             module_dict = {"module": newfile_name, "pod": newfile_name, "git": git, "branch": n_branch}
@@ -1396,7 +1430,7 @@ class yhgit:
             printTable(fail_list, "realse失败的模块")
 
         if os.path.exists(modules_path):
-            del_file(modules_path)
+            del_dir(modules_path)
 
     # 自动提交子模块的代码
     def clean(self):
@@ -1410,10 +1444,45 @@ class yhgit:
             os.remove(local_yaml_path)
         modules_path = "modules/"
         if os.path.exists(modules_path):
-            del_file(modules_path)
+            del_dir(modules_path)
 
         debugInfo("执行成功")
 
+    def help(self):
+        table = PrettyTable(["编号", "指令", "参数", "说明", "使用"])
+        table.add_row(
+            [0, "init", "-b 指定分支名; 组件git地址",
+             "根据git地址，新建分支，然后自动更新PodfileModule.yaml及PodfileLocal.yaml文件",
+             "yhgit init -b test git@gitlab.yonghui.cn:operation-xm-qdjg/yhfoundation-ios.git"])
+        table.add_row(
+            [0, "install", "-b 指定分支名; 组件 指定 新建分支的组件",
+             "根据分支branch，然后基于组件或者PodfileLocal.yaml中的配置，新建分支branch",
+             "yhgit install -b test abc \n yhgit install"])
+        table.add_row(
+            [1, "status", "", "根据本地PodfileLocal.yaml中配置的仓库，获取所有仓库的开发分支及仓库状态", "yhgit status"])
+        table.add_row([2, "commit", "-m 指定提交的信息", "根据本地PodfileLocal.yaml中配置的仓库，提交本地仓库中的修改",
+                       "yhgit commit -m \'提交信息\'"])
+        table.add_row([3, "pull", "", "根据本地PodfileLocal.yaml中配置的仓库，拉取远端仓库中代码", "yhgit pull"])
+        table.add_row([4, "push", "", "根据本地PodfileLocal.yaml中配置的仓库，推送本地代码到远端", "yhgit push"])
+        table.add_row(
+            [5, "merge", "-b 指定分支名; 组件 指定 merge的组件", "然后基于组件merge 分支到当前开发分支，并自动提交",
+             "yhgit merge -b master abc"])
+        table.add_row([6, "release", "组件： 指定组件",
+                       "根据本地PodfileModule.yaml中配置的仓库或者手动指定仓库，合并开发分支到master，并自动打新的tag",
+                       "yhgit release abc"])
+        table.add_row([7, "clean", "", "清空本地PodfileLocal.yaml 及 modules文件夹", "yhgit clean"])
+        table.add_row([8, "version", "", "查看版本号", "yhgit version \n yhgit -v"])
+        table.add_row([8, "help", "", "查看帮助文档", "yhgit help \n yhgit -h"])
+
+        table.align[1] = '1'
+        table.border = True
+        table.junction_char = '+'
+        table.horizontal_char = '-'
+        table.vertical_char = '|'
+        table.hrules = ALL
+        debugInfo(table)
+        debugInfo("\n")
+
 
 def debugInfo(msg):
     logging.info(msg)
@@ -1426,6 +1495,9 @@ def main(argvs=None):
     if argvs is None:
         argvs = sys.argv
     # 获取指令类型
+    if len(argvs) <= 1:
+        yhgit().help()
+        return
     command = argvs[1]
     branchname = ''
     modules = []
@@ -1531,40 +1603,7 @@ def main(argvs=None):
     elif command == "version" or command == "-v":
         debugInfo(pkg_resources.require("yhgit")[0].version)
     elif command == "help" or command == "-h":
-
-        table = PrettyTable(["编号", "指令", "参数", "说明", "使用"])
-        table.add_row(
-            [0, "init", "-b 指定分支名; 组件git地址",
-             "根据git地址，新建分支，然后自动更新PodfileModule.yaml及PodfileLocal.yaml文件",
-             "yhgit init -b test git@gitlab.yonghui.cn:operation-xm-qdjg/yhfoundation-ios.git"])
-        table.add_row(
-            [0, "install", "-b 指定分支名; 组件 指定 新建分支的组件",
-             "根据分支branch，然后基于组件或者PodfileLocal.yaml中的配置，新建分支branch",
-             "yhgit install -b test abc \n yhgit install"])
-        table.add_row(
-            [1, "status", "", "根据本地PodfileLocal.yaml中配置的仓库，获取所有仓库的开发分支及仓库状态", "yhgit status"])
-        table.add_row([2, "commit", "-m 指定提交的信息", "根据本地PodfileLocal.yaml中配置的仓库，提交本地仓库中的修改",
-                       "yhgit commit -m \'提交信息\'"])
-        table.add_row([3, "pull", "", "根据本地PodfileLocal.yaml中配置的仓库，拉取远端仓库中代码", "yhgit pull"])
-        table.add_row([4, "push", "", "根据本地PodfileLocal.yaml中配置的仓库，推送本地代码到远端", "yhgit push"])
-        table.add_row(
-            [5, "merge", "-b 指定分支名; 组件 指定 merge的组件", "然后基于组件merge 分支到当前开发分支，并自动提交",
-             "yhgit merge -b master abc"])
-        table.add_row([6, "release", "组件： 指定组件",
-                       "根据本地PodfileModule.yaml中配置的仓库或者手动指定仓库，合并开发分支到master，并自动打新的tag",
-                       "yhgit release abc"])
-        table.add_row([7, "clean", "", "清空本地PodfileLocal.yaml 及 modules文件夹", "yhgit clean"])
-        table.add_row([8, "version", "", "查看版本号", "yhgit version \n yhgit -v"])
-        table.add_row([8, "help", "", "查看帮助文档", "yhgit help \n yhgit -h"])
-
-        table.align[1] = '1'
-        table.border = True
-        table.junction_char = '+'
-        table.horizontal_char = '-'
-        table.vertical_char = '|'
-        table.hrules = ALL
-        debugInfo(table)
-        debugInfo("\n")
+        yhgit().help()
 
 
 if __name__ == '__main__':
